﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using ServiceSecurity;
using ServiceSecurity.Data.Enumerations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OWL.ServiceSecurity.Middleware
{
    /// <summary>
    /// Middleware that is called before an endpoint is called. Checks if a valid API Key is provided.
    /// </summary>
    public class ApiKeyMiddleware
    {
        private readonly RequestDelegate next;
        private const string HttpKeyHeader = "x-api-key";
        private const string SwaggerPath = "/swagger";
        private eServiceNames _serviceName;

        public ApiKeyMiddleware(RequestDelegate next, eServiceNames serviceName)
        {
            this.next = next;
            _serviceName = serviceName;
        }

        public async Task InvokeAsync(HttpContext context)
        {
            var endpoint = context.Features.Get<IEndpointFeature>();

            if (endpoint != null)
            {
                if(endpoint.Endpoint?.Metadata?.GetMetadata<IAllowAnonymous>() is object)
                {
                    await next.Invoke(context);
                }
            }
            else if (_serviceName.Equals(eServiceNames.DataSyncImport))
            {
                if (AllowElsaSiteRequest(context.Request.Path))
                {
                    await next.Invoke(context);
                }
            }
            else if (context.Request.Headers.ContainsKey("Authorization"))
            {
                await next.Invoke(context);
            }
            else if(context.Request.Path.Value.Equals(SwaggerPath) || context.Request.Path.Value.Contains("ValidateApiKey"))
            {
                Console.WriteLine("No Auth Header detected. Continuing");

                await next.Invoke(context);
            } 
            else
            {
                if (!context.Request.Headers.TryGetValue(HttpKeyHeader, out var extractedApiKey))
                {
                    context.Response.StatusCode = 401;
                    await context.Response.WriteAsync("Api Key was not provided. (Using ApiKeyMiddleware) ");
                    return;
                }

                // Check with storage Account
                var keyIsValid = await new KeyChecker().IsKeyValid(extractedApiKey, _serviceName.ToString());

                if (keyIsValid)
                {
                    await next.Invoke(context);
                }
                else
                {
                    context.Response.StatusCode = 401;
                    await context.Response.WriteAsync("Provided Api Key is not valid. (Using ApiKeyMiddleware) ");
                    return;
                }
            }
        }

        private bool AllowElsaSiteRequest(string path)
        {
            if (path.Contains("/parser/")) return false;
            else return true;
        }
    }
}
