﻿using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using ServiceSecurity.Data;
using ServiceSecurity.Data.Enumerations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace ServiceSecurity
{
    public class OwlHttpClient
    {
        private readonly ApiKeyOptions _apiKeyConfiguration;
        private readonly string KeyAuthHeader = "x-api-key";

        public OwlHttpClient(IOptions<ApiKeyOptions> apiKeyConfig)
        {
            _apiKeyConfiguration = apiKeyConfig.Value;
        }

        /// <summary>
        /// Creates an Http client with a keycloak token authentication, as well as optional request headers. 
        /// </summary>
        /// <param name="baseUrl">The main url without detailed path</param>
        /// <param name="token">THe keycloak user token</param>
        /// <param name="optionalHeaders">Optional Http Headers wich will be added to the client by default if they are provided</param>
        /// <returns></returns>
        public HttpClient GetTokenClient(string baseUrl, string token, HttpRequestHeaders? optionalHeaders = null)
        {
            var client = new HttpClient();
            client.BaseAddress = new Uri(baseUrl);

            if (optionalHeaders != null)
            {
                client.DefaultRequestHeaders.Concat(optionalHeaders);
            }

            var authHeader = new AuthenticationHeaderValue("Bearer", token);
            client.DefaultRequestHeaders.Authorization = authHeader;

            return client;
        }

        /// <summary>
        /// Creates an Http client with an Api Key authentication, as well as optional request headers. 
        /// </summary>
        /// <param name="baseUrl">The main url without detailed path</param>
        /// <param name="serviceName">The service for which the Api Key should be used</param>
        /// <param name="optionalHeaders">Optional Http Headers wich will be added to the client by default if they are provided</param>
        /// <returns></returns>
        public HttpClient GetKeyClient(string baseUrl, eServiceNames serviceName, HttpRequestHeaders? optionalHeaders = null)
        {
            var client = new HttpClient();
            client.BaseAddress = new Uri(baseUrl);

            if(optionalHeaders != null)
            {
                client.DefaultRequestHeaders.Concat(optionalHeaders);
            }

            var apiKey = GetMappedApiKey(serviceName);
            client.DefaultRequestHeaders.Add(KeyAuthHeader, apiKey);

            return client;
        }

        private string GetMappedApiKey(eServiceNames serviceName)
        {
            switch (serviceName)
            {
                case eServiceNames.DataSyncImport:
                    return _apiKeyConfiguration.ElsaWorkflowHostKey;
                case eServiceNames.EventPortalDomain:
                    return _apiKeyConfiguration.EventPortalDomainKey;
                case eServiceNames.EventManagementDomain:
                    return _apiKeyConfiguration.EventManagementDomainKey;
                case eServiceNames.PersonDataManagement:
                    return _apiKeyConfiguration.PersonDataManagementKey;
                case eServiceNames.TravelTimeService:
                    return _apiKeyConfiguration.TravelTimeServiceKey;
                default:
                    return string.Empty;
            }
        }
    }
}
